package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import com.bpcl.lpg.core.model.LPGNextApiCredsModel;
import java.util.Base64;
import java.util.Objects;
import com.bpcl.lpg.core.model.OperatorCODRefillsModel;
import java.util.Optional;
import java.io.IOException;
import java.net.http.HttpConnectTimeoutException;
import com.bpcl.core.util.BPCLDateUtils;
import java.util.Date;
import java.net.URI;
import java.net.http.HttpRequest;
import java.time.Duration;
import java.net.http.HttpResponse;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.core.helper.BpclHelper;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import java.net.http.HttpClient;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.BpclHttpClientService;

public class BpclHttpClientServiceImpl implements BpclHttpClientService
{
    private static final Logger LOG;
    private static final String RESPONSE_MSG = "Response data :: {} for the endPoint :: {} took :: {} ms";
    private static final String LPGNEXT_RESPONSE_MSG = "LPGNEXT Response data :: {} for the endPoint :: {} took :: {} ms correlationID :: {} and timestamp {}";
    private static final String RESPONSE_MSG_SAFETY_INSPECTION = "Response data :: {} for the endPoint :: {} took :: {} ms and CorrelationID :: {}";
    private static final String REQUEST_MSG = "Request data :: {} :: for the end point :: {} ";
    private static final String CORRELATION_HEADER_KEY = "X-BPCL-Correlation-Id";
    private static final HttpClient httpClient;
    private LPGCredentialsService lpgCredentialsService;
    private BpclHelper bpclHelper;
    private ConfigurationService configurationService;
    
    public HttpResponse<String> executePostMethod(final String uri, final String jsonBody, final String apiName) {
        HttpResponse<String> response = null;
        HttpClient client = null;
        if ("confirmRefillPaymentUrl".equals(apiName) || "fetchPaymentDetailsUrl".equals(apiName) || "fetchPaymentStatus".equals(apiName)) {
            client = HttpClient.newHttpClient();
        }
        else {
            final Integer connectionTimeout = this.getConfigurationService().getConfiguration().getInteger("lpgnext.api.connection.timeout", Integer.valueOf(30));
            client = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(connectionTimeout)).build();
        }
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(uri)).header("Content-Type", "application/json").header("Authorization", this.basicAuthorization(apiName)).POST(HttpRequest.BodyPublishers.ofString(jsonBody)).build();
        BpclHttpClientServiceImpl.LOG.info("LPGNXT >>> Request Header : {}", uri);
        if (BpclHttpClientServiceImpl.LOG.isDebugEnabled()) {
            BpclHttpClientServiceImpl.LOG.debug("Request data :: {} :: for the end point :: {} ", jsonBody, uri);
        }
        final long startTime = System.currentTimeMillis();
        try {
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
            final long elapsedTime = System.currentTimeMillis() - startTime;
            String correlationId = null;
            if (response != null) {
                if (null != response.headers()) {
                    final Optional<String> correlation = response.headers().firstValue("X-BPCL-Correlation-Id");
                    if (correlation.isPresent()) {
                        correlationId = correlation.get();
                    }
                }
                BpclHttpClientServiceImpl.LOG.info("LPGNXT >>> Response Header :{}", response.headers());
            }
            BpclHttpClientServiceImpl.LOG.info("LPGNXT >>> Request End Point :{} and Elapsed time :{} and Correlation ID is :{}", uri, elapsedTime, correlationId);
            if (BpclHttpClientServiceImpl.LOG.isDebugEnabled()) {
                BpclHttpClientServiceImpl.LOG.debug("LPGNEXT Response data :: {} for the endPoint :: {} took :: {} ms correlationID :: {} and timestamp {}", (response == null) ? null : response.body(), uri, elapsedTime, correlationId, BPCLDateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"));
            }
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms. Requested url: {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    public HttpResponse<String> executePostMethodWithTimeOut(final String uri, final String jsonBody, final String apiName, final OperatorCODRefillsModel operatorCodRefillBookingModel) {
        HttpResponse<String> response = null;
        final Integer connectionTimeout = this.getConfigurationService().getConfiguration().getInteger("refill.delivery.connection.timeout", Integer.valueOf(30));
        final HttpClient client = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(connectionTimeout)).build();
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(uri)).header("Content-Type", "application/json").header("Authorization", this.basicAuthorization(apiName)).POST(HttpRequest.BodyPublishers.ofString(jsonBody)).build();
        BpclHttpClientServiceImpl.LOG.info("Request data :: {} :: for the end point :: {} ", jsonBody, uri);
        final long startTime = System.currentTimeMillis();
        try {
            operatorCodRefillBookingModel.setReqStartTime(Long.valueOf(System.currentTimeMillis()));
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
            operatorCodRefillBookingModel.setReqEndTime(Long.valueOf(System.currentTimeMillis()));
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms.Requested url: {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    public HttpResponse<String> executePostMethodForSafetyInspection(final String uri, final String jsonBody, final String apiName) {
        HttpResponse<String> response = null;
        String correlationId = null;
        final Integer connectionTimeout = this.getConfigurationService().getConfiguration().getInteger("refill.delivery.connection.timeout", Integer.valueOf(30));
        final HttpClient client = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(connectionTimeout)).build();
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(uri)).header("Content-Type", "application/json").header("Authorization", this.basicAuthorization(apiName)).POST(HttpRequest.BodyPublishers.ofString(jsonBody)).build();
        if (this.getConfigurationService().getConfiguration().getBoolean("enable.safety.inspection.log", false)) {
            BpclHttpClientServiceImpl.LOG.info("Request data :: {} :: for the end point :: {} ", jsonBody, uri);
        }
        final long startTime = System.currentTimeMillis();
        try {
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
            final long elapsedTime = System.currentTimeMillis() - startTime;
            if (response != null) {
                if (null != response.headers()) {
                    final Optional<String> correlation = response.headers().firstValue("X-BPCL-Correlation-Id");
                    if (correlation.isPresent()) {
                        correlationId = correlation.get();
                    }
                }
                BpclHttpClientServiceImpl.LOG.info("LPGNXT >>> Response Header :{}", response.headers());
            }
            if (this.getConfigurationService().getConfiguration().getBoolean("enable.safety.inspection.log", false)) {
                BpclHttpClientServiceImpl.LOG.info("Response data :: {} for the endPoint :: {} took :: {} ms and CorrelationID :: {}", (response == null) ? null : response.body(), uri, elapsedTime, correlationId);
            }
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms. Requested url: {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    public HttpResponse<String> executePostMethodForBDA(final String uri, final String jsonBody, final String apiName) {
        HttpResponse<String> response = null;
        String correlationId = null;
        final Integer connectionTimeout = this.getConfigurationService().getConfiguration().getInteger("refill.delivery.connection.timeout", Integer.valueOf(30));
        final HttpClient client = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(connectionTimeout)).build();
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(uri)).header("Content-Type", "application/json").header("Authorization", this.basicAuthorization(apiName)).POST(HttpRequest.BodyPublishers.ofString(jsonBody)).build();
        if (this.getConfigurationService().getConfiguration().getBoolean("enable.bda.functionality.log", false)) {
            BpclHttpClientServiceImpl.LOG.info("Request data :: {} :: for the end point :: {} ", jsonBody, uri);
        }
        final long startTime = System.currentTimeMillis();
        try {
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
            final long elapsedTime = System.currentTimeMillis() - startTime;
            if (this.getConfigurationService().getConfiguration().getBoolean("enable.bda.functionality.log", false)) {
                if (response != null && null != response.headers()) {
                    final Optional<String> correlation = response.headers().firstValue("X-BPCL-Correlation-Id");
                    if (correlation.isPresent()) {
                        correlationId = correlation.get();
                    }
                }
                BpclHttpClientServiceImpl.LOG.info("LPGNEXT Response data :: {} for the endPoint :: {} took :: {} ms correlationID :: {} and timestamp {}", (response == null) ? null : response.body(), uri, elapsedTime, correlationId, BPCLDateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"));
            }
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms. Requested url: {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    public HttpResponse<String> executeGetMethod(final String uri) {
        HttpResponse<String> response = null;
        final HttpClient client = HttpClient.newHttpClient();
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(uri)).build();
        final long startTime = System.currentTimeMillis();
        try {
            response = client.send(request, HttpResponse.BodyHandlers.ofString());
            final long n = System.currentTimeMillis() - startTime;
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms. Requested url {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    public HttpResponse<String> executePostWoAuthorization(final String uri, final String msg) {
        final HttpRequest.BodyPublisher publisher = HttpRequest.BodyPublishers.ofString(msg);
        final HttpRequest request = HttpRequest.newBuilder(URI.create(uri)).POST(publisher).build();
        HttpResponse<String> response = null;
        if (this.bpclHelper.isDeveloperMode()) {
            BpclHttpClientServiceImpl.LOG.info("Request data :: {} :: for the end point :: {} ", msg, uri);
        }
        final long startTime = System.currentTimeMillis();
        try {
            response = BpclHttpClientServiceImpl.httpClient.send(request, HttpResponse.BodyHandlers.ofString());
            final long elapsedTime = System.currentTimeMillis() - startTime;
            if (this.bpclHelper.isDeveloperMode()) {
                BpclHttpClientServiceImpl.LOG.info("Response data :: {} for the endPoint :: {} took :: {} ms", (response == null) ? null : response.body(), uri, elapsedTime);
            }
        }
        catch (final HttpConnectTimeoutException connectTimeoutException) {
            BpclHttpClientServiceImpl.LOG.error("LPGNext connection timed out after {} ms. Requested url {}", (Object)(System.currentTimeMillis() - startTime), uri);
            throw new RuntimeException("Request Timed Out. Please Try Again");
        }
        catch (final IOException | InterruptedException e) {
            BpclHttpClientServiceImpl.LOG.error("Interrupted exception while executing client", e.getCause());
            throw new RuntimeException("Timed Out. Please Try Again");
        }
        catch (final Exception e) {
            BpclHttpClientServiceImpl.LOG.error("Exception occurred execute post method with timeout {}", e.getCause());
            throw new RuntimeException("Something went wrong please try again");
        }
        return response;
    }
    
    private String basicAuthorization(final String apiName) {
        Objects.requireNonNull(apiName, "apiName should not be null");
        final LPGNextApiCredsModel credsModel = this.lpgCredentialsService.getCredsModelByApiName(apiName);
        final String userName = credsModel.getUsername();
        Objects.requireNonNull(userName, "Username should not be null");
        Objects.requireNonNull(credsModel.getPassword(), "Password should not be null");
        return "Basic " + Base64.getEncoder().encodeToString(userName.concat(":").concat(credsModel.getPassword()).getBytes());
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    public void setBpclHelper(final BpclHelper bpclHelper) {
        this.bpclHelper = bpclHelper;
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(BpclHttpClientServiceImpl.class);
        httpClient = HttpClient.newBuilder().version(HttpClient.Version.HTTP_2).build();
    }
}
